from typing import Any, Callable, ClassVar, List, Optional, Tuple, TypeVar, Union

FieldTypePlaceholder = Any
T = TypeVar("T", bound="SecureObject")

runtime = None

class SecureObject:
    __slots__ = "share"
    def __init__(self, value: Optional[Any] = ...) -> None: ...

class SecureNumber(SecureObject):
    __slots__ = ()

    bit_length: ClassVar[Optional[int]] = None
    def __neg__(self: T) -> T: ...
    def __pow__(self: T, other: int) -> T: ...
    def __pos__(self: T) -> T: ...
    def __abs__(self: T) -> T: ...
    def __add__(self: T, other: Union[T, int, float]) -> T: ...
    def __radd__(self: T, other: Union[T, int, float]) -> T: ...
    def __sub__(self: T, other: Union[T, int, float]) -> T: ...
    def __rsub__(self: T, other: Union[T, int, float]) -> T: ...
    def __mul__(self: T, other: Union[T, int, float]) -> T: ...
    def __rmul__(self: T, other: Union[T, int, float]) -> T: ...
    def __truediv__(self: T, other: Union[T, int, float]) -> T: ...
    def __rtruediv__(self: T, other: Union[T, int, float]) -> T: ...
    def __mod__(self: T, other: Union[T, int]) -> T: ...
    def __rmod__(self: T, other: Union[T, int]) -> T: ...
    def __floordiv__(self: T, other: int) -> T: ...
    def __rfloordiv__(self: T, other: int) -> T: ...
    def __divmod__(self: T, other: int) -> Tuple[T, T]: ...
    def __rdivmod__(self: T, other: int) -> Tuple[T, T]: ...
    def __lshift__(self: T, other: int) -> T: ...
    def __rlshift__(self: T, other: int) -> T: ...
    def __rshift__(self: T, other: int) -> T: ...
    def __rrshift__(self: T, other: int) -> T: ...
    def __and__(self: T, other: Union[int, T]) -> T: ...
    def __rand__(self: T, other: Union[int, T]) -> T: ...
    def __xor__(self: T, other: Union[int, T]) -> T: ...
    def __rxor__(self: T, other: Union[int, T]) -> T: ...
    # still missing invert, or, ror, lt, le, eq, ge, gt and ne

class SecureFiniteField(SecureNumber):
    __slots__ = ()

    frac_length: ClassVar[Optional[int]] = 0
    field: Optional[FieldTypePlaceholder] = None

    _output_conversion: ClassVar[
        Optional[Union[type, Callable[[Union[int, float]], Union[int, float]]]]
    ] = None
    def __init__(
        self, value: Optional[Union[int, FieldTypePlaceholder]] = None
    ) -> None: ...

class SecureInteger(SecureNumber):
    __slots__ = ()

    frac_length: ClassVar[int] = 0
    field: Optional[FieldTypePlaceholder] = None

    _output_conversion: ClassVar[
        Union[type, Callable[[Union[int, float]], Union[int, float]]]
    ] = int
    def __init__(
        self, value: Optional[Union[int, FieldTypePlaceholder]] = None
    ) -> None: ...

class SecureFixedPoint(SecureNumber):
    """Base class for secure (secret-shared) fixed-point numbers."""

    __slots__ = "integral"

    frac_length: ClassVar[int] = 0
    field: Optional[FieldTypePlaceholder] = None
    @classmethod
    def _output_conversion(cls, a: Union[int, float]) -> float: ...
    def __init__(
        self,
        value: Optional[Union[int, float, FieldTypePlaceholder]] = None,
        integral: Optional[bool] = None,
    ) -> None: ...

def SecFld(
    order: Optional[int] = None,
    modulus: Optional[Union[int, str, Any]] = None,
    char: Optional[int] = None,
    ext_deg: Optional[int] = None,
    min_order: Optional[int] = None,
    signed: bool = False,
) -> SecureFiniteField: ...
def SecInt(
    l: Optional[int] = None, p: Optional[int] = None, n: int = 2
) -> SecureInteger: ...
def SecFxp(
    l: Optional[int] = None,
    f: Optional[int] = None,
    p: Optional[int] = None,
    n: int = 2,
) -> SecureFixedPoint: ...

class SecureFloat(SecureNumber):
    __slots__ = ()

    significand_type: Optional[SecureObject] = None
    exponent_type: Optional[SecureObject] = None
    def __init__(self, value: Optional[Union[int, float]] = None) -> None: ...
    @classmethod
    def _input(
        cls, x: List[SecureFloat], senders: List[int]
    ) -> List[List[SecureFloat]]: ...

def SecFlt(
    l: Optional[int] = None, s: Optional[int] = None, e: Optional[int] = None
) -> SecureFloat: ...
